<?php

class Onlinebiz_Optimize_Helper_Data extends Mage_Core_Helper_Abstract
{

  const XML_SYSTEM_PATH_CONFIG_URL_CLOUDINARY = 'onlinebiz_optimize/cloudmedia/';
  const XML_SYSTEM_PATH_CONFIG_GENERAL = 'onlinebiz_optimize/general/';
  const XML_SYSTEM_PATH_CONFIG_URL_ADAPTIVE = 'onlinebiz_optimize/adaptive/';

  const CLOUDINARY_RAW_UPLOAD_VARIABLE = '{{cloudmedia_raw_upload_url}}';
  const CLOUDINARY_IMAGE_UPLOAD_VARIABLE = '{{cloudmedia_image_upload_url}}';
  const CLOUDINARY_IMAGE_OPTIONS_VARIABLE = '{{cloudmedia_image_upload_options}}';
  const XML_SYSTEM_PATH_COMPRESSION = 'onlinebiz_optimize/general/{{type}}compression';
  const XML_SYSTEM_PATH_IS_FILE_VERSION = 'onlinebiz_optimize/general/version';

  const POSTION_TOP = 'top';
  const POSITION_BOTTOM = 'bottom';
  const POSITION_CENTER = 'center';

  /**
   * Compare param $version with magento version
   *
   * @param string $version Version to compare
   * @param string $operator
   *
   * @return boolean
   */
  public function checkVersion($version, $operator = '>=')
  {
    return version_compare(Mage::getVersion(), $version, $operator);
  }

  /**
   * @param mixed $storeId
   * @return string
   */
  public function isFilesVersion($storeId = null)
  {
    if (Mage::app()->getStore()->isAdmin())
      return false;
    return (int)$this->getCloudinaryConfigData('version', $storeId);
  }

  /**
   * @param string $field
   * @param mixed $storeId
   * @return array
   */
  public function getCloudinaryConfigData($field, $storeId = null)
  {
    $path = self::XML_SYSTEM_PATH_CONFIG_URL_CLOUDINARY . $field;
    return Mage::getStoreConfig($path, $storeId);
  }

  /**
   * @param string $field
   * @param mixed $storeId
   * @return array
   */
  public function getGeneralConfig($field, $storeId = null)
  {
    $path = self::XML_SYSTEM_PATH_CONFIG_GENERAL . $field;
    return Mage::getStoreConfig($path, $storeId);
  }

  /**
   * @param mixed $storeId
   * @return string
   */
  public function getIMGixOptions($storeId = null)
  {
    if (!$this->isCloudinaryActive())
      return null;
    return trim(Mage::helper('core')->htmlEscape((string)$this->getCloudinaryConfigData('options', $storeId)));
  }

  /**
   * @param mixed $storeId
   * @return string
   */
  public function isCloudinaryActive($storeId = null)
  {
    return (int)$this->getCloudinaryConfigData('enable', $storeId);
  }

  /**
   * @param mixed $storeId
   * @return string
   */
  public function getIMGixAdvancedOptions($storeId = null)
  {
    if (!$this->isCloudinaryActive())
      return null;
    $advancedOptions = trim(Mage::helper('core')->htmlEscape((string)$this->getCloudinaryConfigData('advanced_options', $storeId)));
    if(!$advancedOptions){
      $advancedOptions = $this->getIMGixOptions($storeId);
    }

    return $advancedOptions;
  }

  /**
   * @param mixed $storeId
   * @return string
   */
  public function isDisabledIMGCache($storeId = null)
  {
    if (!$this->isCloudinaryActive())
      return false;
    return (int)$this->getCloudinaryConfigData('cached', $storeId);
  }

  /**
   * @param mixed $storeId
   * @return string
   */
  public function getURLSkinMerge($secure = true, $storeId = null)
  {
    if (!$this->isCloudinaryActive() || Mage::app()->getStore()->isAdmin()) {
      return Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_SKIN);
    }
    $value = $this->getWebConfigData('skin_merge_url', $secure, $storeId);
    return $this->replaceCloudinaryVariable($value, $storeId);
  }

  /**
   * @param string $field
   * @param mixed $storeId
   * @return array
   */
  public function getWebConfigData($field, $secure = true, $storeId = null)
  {
    $path = 'web/' . ($secure ? 'secure' : 'unsecure') . '/' . $field;
    return Mage::getStoreConfig($path, $storeId);
  }

  /**
   * @param mixed $storeId
   * @return string
   */
  public function replaceCloudinaryVariable($value, $storeId = null)
  {
    if (strpos($value, self::CLOUDINARY_RAW_UPLOAD_VARIABLE) !== false) {
      $rawUrl = $this->getCloudinaryRawURL($storeId);
      $value = str_replace(self::CLOUDINARY_RAW_UPLOAD_VARIABLE, $rawUrl, $value);

    } elseif (strpos($value, self::CLOUDINARY_IMAGE_UPLOAD_VARIABLE) !== false) {
      $imageUrl = $this->getCloudinaryImageURL($storeId);
      $value = str_replace(self::CLOUDINARY_IMAGE_UPLOAD_VARIABLE, $imageUrl, $value);
    }

    if (strpos($value, self::CLOUDINARY_IMAGE_OPTIONS_VARIABLE) !== false) {
      $imageOptions = $this->getCloudinaryImageOptions($storeId);
      $value = str_replace(self::CLOUDINARY_IMAGE_OPTIONS_VARIABLE, $imageOptions, $value);
    }

    return $value;
  }

  /**
   * @param mixed $storeId
   * @return string
   */
  public function getCloudinaryRawURL($storeId = null)
  {
    return (string)$this->getCloudinaryConfigData('raw_upload', $storeId);
  }

  /**
   * @param mixed $storeId
   * @return string
   */
  public function getCloudinaryImageURL($storeId = null)
  {
    return (string)$this->getCloudinaryConfigData('image_upload', $storeId);
  }

  /**
   * @param mixed $storeId
   * @return string
   */
  public function getCloudinaryImageOptions($storeId = null)
  {
    return (string)$this->getCloudinaryConfigData('image_options', $storeId);
  }

  /**
   * @param mixed $storeId
   * @return string
   */
  public function getURLMediaMerge($secure = true, $storeId = null)
  {
    if (!$this->isCloudinaryActive() || Mage::app()->getStore()->isAdmin())
      return Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_MEDIA);
    $value = $this->getWebConfigData('media_merge_url', $secure, $storeId);
    return $this->replaceCloudinaryVariable($value, $storeId);
  }

  public function isFontsFile($filename)
  {
    $fontsExtensions = array('eot', 'woff', 'ttf', 'svg', 'woff2');
    $filename = parse_url($filename);
    $ext = pathinfo($filename['path'], PATHINFO_EXTENSION);
    if (in_array($ext, $fontsExtensions))
      return true;
    else
      return false;
  }

  public function replaceCloudWpImageUrl($content)
  {
    if (!$this->isCloudinaryActive())
      return $content;

    $wpUrl = Mage::helper('wordpress')->getBaseUrl();
    $urlPath = parse_url($wpUrl);
    $wpPath = $urlPath['path'];
    $urlCloud = str_replace('/skin/', $wpPath, Mage::getBaseUrl('skin'));
    return str_replace($wpUrl, $urlCloud, $content);

  }

  public function getCssJsCloudUrl($storeId = null)
  {
    return Mage::getStoreConfig(Onlinebiz_Optimize_Model_Core_Store::XML_PATH_CLOUDINARY . 'basejscss', $storeId);
  }

  public function _getFileVersion($baseDirFile, $type = 'media')
  {
    if (!$this->isFileVersion())
      return;

    $baseDir = ($type == 'js') ? Mage::getBaseDir() . DS . 'js' . DS : Mage::getBaseDir($type) . DS;
    $file = $baseDir . trim($baseDirFile);
    return file_exists($file) ? '?v=' . filemtime($file) : '?v=na';
  }

  /**
   * @param is use file version
   * @param mixed $store
   * @param string $type
   * @return bool
   */
  public function isFileVersion()
  {
    return (int)Mage::getStoreConfig(self::XML_SYSTEM_PATH_IS_FILE_VERSION);
  }

  /**
   * Retreive some basic information depending on the type of file: original path, destination path, type, name, static
   *
   * @param string $name
   * @param string $type
   * @param bool $static
   * @return array $info
   */
  public function getCompressedInfo($name, $type = 'css', $static = false)
  {
    $info = array();
    $designPackage = Mage::getDesign();

    $orgname = $name;
    if (!$static) {
      $info['orgskin_path'] = $designPackage->getFilename($orgname, array('_type' => 'skin'));
    } else {
      $info['orgskin_path'] = Mage::getBaseDir() . DS . 'js' . DS . $name;
    }

    if (isset($info['orgskin_path']) && !file_exists($info['orgskin_path']) || !filesize($info['orgskin_path']))
      return false;

    $orgpath = explode(DS, $info['orgskin_path']);
    if (count($orgpath) > 1 && !$static) {
      // Get the theme of the original file
      $info['theme_name'] = $orgpath [count($orgpath) - (count(explode('/', $orgname)) + 1)];
    }
    $name = $this->_getFileNameFromPath($name);
    $name = !$name ? substr($name, 0, strpos($name, '.' . $type)) : $name;
    $name =  md5($info['orgskin_path']) . '_' . $name;
    $file_path = explode('/', $name);
    if (count($file_path) > 1) {
      $name = array_pop($file_path);
      $info['file_path'] = implode(DS, $file_path);
    } else {
      $info['file_path'] = '';
    }
    $info['original_name'] = $orgname;
    $info['new_name'] = $name;
    $info['type'] = $type;
    $info['static'] = $static;

    $targetDir = $this->initMergerDir($type);
    if (!$targetDir) {
      $info['success'] = false;
      return $info;
    }


    if (Mage::app()->getStore()->isCurrentlySecure() && $type == 'css') {
      $info['targetPath'] = Mage::getBaseDir('media') . DS . 'css_secure' . DS;
    } else {
      $info['targetPath'] = Mage::getBaseDir('media') . DS . $type . DS;
    }

    $info['targetPathFile'] = $info['targetPath'] . $name;

    if (!file_exists($info['targetPath'])) {
      mkdir($info['targetPath'], 0755, true);
    }

    return $info;
  }

  /**
   * Make sure merger dir exists and writeable
   * Also can clean it up
   *
   * @param string $dirRelativeName
   * @param bool $cleanup
   * @return bool|string
   */
  public function initMergerDir($dirRelativeName, $cleanup = false)
  {
    $mediaDir = Mage::getBaseDir('media');
    try {
      $dir = Mage::getBaseDir('media') . DS . $dirRelativeName;
      if ($cleanup) {
        Diglin_Io_File::rmdirRecursive($dir);
      }
      if (!is_dir($dir)) {
        mkdir($dir);
      }
      return is_writeable($dir) ? $dir : false;
    } catch (Exception $e) {
      Mage::logException($e);
    }
    return false;
  }

  /**
   * Provide the path URL of the filesystem for js or css files compressed or not
   *
   * @param array $info
   * @param string|array $mergeCallback
   * @return string
   */
  public function getResultPath($info, $mergeCallback)
  {
    $designPackage = Mage::getDesign();
    // Skin items
    if (!$info['result']) { // Default Folder
      return ($mergeCallback ? $designPackage->getFilename($info['original_name'], array('_type' => 'skin')) : $designPackage->getSkinUrl($info['original_name'], array()));
    } else {
      $type = $info['type'];
      $version = $this->isFileVersion();
      $time = '';
      $isSecure = Mage::app()->getStore()->isCurrentlySecure();
      if ($version) {
        $time = '?v=' . filectime($info['targetPathFile']);
      }
      $baseFilePath = ($isSecure && $type == 'css') ? 'css_secure' . '/' . $info['new_name'] : $type . '/' . $info['new_name'];
      $mediaMerge = $this->getMediaMergeUrl($baseFilePath, Mage::app()->getStore()->getId());

      if ($isSecure && $type == 'css') {
        return $mediaMerge . 'css_secure' . '/' . $info['new_name'] . $time;
      } else {
        return $mediaMerge . $type . '/' . $info['new_name'] . $time;
      }
    }
  }

  public function getMediaMergeUrl($baseDirFile, $storeId = null)
  {
    return Mage::getBaseUrl(Onlinebiz_Optimize_Model_Core_Store::URL_TYPE_MEDIA_MERGE);
  }

  /**
   * Add trailing slash if necessary
   *
   * @param string $url
   */
  public function getTrailingSlash($url)
  {
    if (Mage::getStoreConfig('web/seo/trailingslash')) {
      if (!preg_match('/\\.(rss|html|htm|xml|php?)$/', strtolower($url)) && substr($url, -1) != '/') {
        $url .= '/';
      }
    }
    return $url;
  }

  /**
   * @param is use csscompression
   * @param mixed $store
   * @return bool
   */
  public function isUseCompression($type = 'css', $store = null)
  {
    $path = str_replace('{{type}}', $type, self::XML_SYSTEM_PATH_COMPRESSION);
    return (int)Mage::getStoreConfig($path, $store);
  }

  public function getCropPositionsOptions()
  {
    return array(
      self::POSTION_TOP => $this->__('Top image'),
      self::POSITION_CENTER => $this->__('Center image'),
      self::POSITION_BOTTOM => $this->__('Bottom image')
    );
  }

  public function _getCropPosition()
  {
    return (int)$this->getAdaptiveConfigData('position');
  }

  /**
   * @param string $field
   * @param mixed $storeId
   * @return array
   */
  public function getAdaptiveConfigData($field, $storeId = null)
  {
    $path = self::XML_SYSTEM_PATH_CONFIG_URL_ADAPTIVE . $field;
    return Mage::getStoreConfig($path, $storeId);
  }

  public function useAdaptive()
  {
    return (int)$this->getAdaptiveConfigData('enable');
  }

  public function isDevMode()
  {
    return (int)$this->getCloudinaryConfigData('devmode') && $this->isCloudinaryActive();
  }

  public function _getImageSrcset($imgUrl)
  {
    if (!$imgUrl || !$this->isCloudinaryActive())
      return;
    $html = 'srcset="%s" ';
    $urlSet = array();
    for ($i = 1; $i < 4; $i++) {
      $urlSet[] = $imgUrl . '&dpr=' . $i . ' ' . $i . 'x';
    }
    return sprintf($html, implode(',', $urlSet));
  }

  private function _getFileNameFromPath($filePath)
  {
    try {
      $filePathInfo = explode('/', $filePath);
      return array_pop($filePathInfo);
    } catch (Exception $e) {
      return false;
    }
  }

  public function isUrlsBlacklist()
  {
    $config = $this->getGeneralConfig('url_blacklist');
    $urls = explode(PHP_EOL, $config);
    if (count($urls)){
      $urls = array_unique(array_filter($urls));
      $requestUri = Mage::app()->getRequest()->getRequestUri();
      foreach ($urls as $url){
        if(strpos($requestUri, $url) !== false) {
          return true;
        }
      }
    }
    return false;
  }

}
