<?php
/**
 * Magento
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@magentocommerce.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Magento to newer
 * versions in the future. If you wish to customize Magento for your
 * needs please refer to http://www.magentocommerce.com for more information.
 *
 * @category    Mage
 * @package     Mage_Catalog
 * @copyright   Copyright (c) 2012 Magento Inc. (http://www.magentocommerce.com)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

/**
 * Catalog image helper
 *
 * @author      Magento Core Team <core@magentocommerce.com>
 */
class Onlinebiz_Optimize_Helper_Image extends Onlinebiz_Optimize_Helper_Image_Abstract
{
  /**
   * Used Advanced options
   *
   * @var int
   */
  protected $_useAdvanced = false;

  /**
   * Reset all previous data
   *
   * @return Mage_Catalog_Helper_Image
   */
  protected function _reset()
  {
    parent::_reset();
    $this->_useAdvanced = false;
    return $this;
  }

  /**
   * Initialize Helper to work with Image
   *
   * @param Mage_Catalog_Model_Product $product
   * @param string $attributeName
   * @param mixed $imageFile
   * @return Mage_Catalog_Helper_Image
   */
  public function init(Mage_Catalog_Model_Product $product, $attributeName, $imageFile = null, $useAdvanced = false)
  {
    parent::init($product, $attributeName, $imageFile, $useAdvanced);
    $this->_useAdvanced = $useAdvanced;
    return $this;
  }

  /**
   * Return Image URL
   *
   * @return string
   */
  public function __toString()
  {
    if (Mage::app()->getStore()->isAdmin()) {
      return parent::__toString();
    }
    try {
      $model = $this->_getModel();

      if ($this->getImageFile()) {
        $model->setBaseFile($this->getImageFile(), $this->_useAdvanced);
      } else {
        $model->setBaseFile($this->getProduct()->getData($model->getDestinationSubdir()), $this->_useAdvanced);
      }
      $disableIMGCache = Mage::helper('onlinebiz_optimize')->isDisabledIMGCache();

      if ($model->isCached() || $disableIMGCache) {
        return $model->getUrl();
      } else {
        if ($this->_scheduleRotate) {
          $model->rotate($this->getAngle());
        }

        if ($this->_scheduleResize) {
          $model->resize();
        }

        if ($this->getWatermark()) {
          $model->setWatermark($this->getWatermark());
        }
        $url = $model->saveFile()->getUrl();
      }
    } catch (Exception $e) {
      $url = Mage::getDesign()->getSkinUrl($this->getPlaceholder());
    }
    return $url;
  }

}
