<?php
/**
 * Magento
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@magentocommerce.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Magento to newer
 * versions in the future. If you wish to customize Magento for your
 * needs please refer to http://www.magentocommerce.com for more information.
 *
 * @category    Mage
 * @package     Mage_Core
 * @copyright   Copyright (c) 2012 Magento Inc. (http://www.magentocommerce.com)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

/**
 * Store model
 *
 * @method Mage_Core_Model_Resource_Store _getResource()
 * @method Mage_Core_Model_Resource_Store getResource()
 * @method Mage_Core_Model_Store setCode(string $value)
 * @method Mage_Core_Model_Store setWebsiteId(int $value)
 * @method Mage_Core_Model_Store setGroupId(int $value)
 * @method Mage_Core_Model_Store setName(string $value)
 * @method int getSortOrder()
 * @method Mage_Core_Model_Store setSortOrder(int $value)
 * @method Mage_Core_Model_Store setIsActive(int $value)
 *
 * @category    Mage
 * @package     Mage_Core
 * @author      Magento Core Team <core@magentocommerce.com>
 */
class Onlinebiz_Optimize_Model_Core_Store extends Mage_Core_Model_Store
{
  /**
   * Possible URL types
   */
  const URL_TYPE_SKIN_BASE = 'skinbase';
  const URL_TYPE_SKIN_CLOUD = 'skincloud';
  const URL_TYPE_SKIN_MERGE = 'skin_merge';
  const URL_TYPE_MEDIA_BASE = 'mediabase';
  const URL_TYPE_MEDIA_MERGE = 'media_merge';

  const XML_PATH_CLOUDINARY = 'onlinebiz_optimize/cloudmedia/';

  /**
   * Retrieve base URL
   *
   * @param string $type
   * @param boolean|null $secure
   * @return string
   */
  public function getBaseUrl($type = self::URL_TYPE_LINK, $secure = null)
  {
    $cacheKey = $type . '/' . (is_null($secure) ? 'null' : ($secure ? 'true' : 'false'));
    if (!isset($this->_baseUrlCache[$cacheKey])) {
      switch ($type) {
        case self::URL_TYPE_WEB:
          $secure = is_null($secure) ? $this->isCurrentlySecure() : (bool)$secure;
          $url = $this->getConfig('web/' . ($secure ? 'secure' : 'unsecure') . '/base_url');
          break;

        case self::URL_TYPE_LINK:
          $secure = (bool)$secure;
          $url = $this->getConfig('web/' . ($secure ? 'secure' : 'unsecure') . '/base_link_url');
          $url = $this->_updatePathUseRewrites($url);
          $url = $this->_updatePathUseStoreView($url);
          break;

        case self::URL_TYPE_DIRECT_LINK:
          $secure = (bool)$secure;
          $url = $this->getConfig('web/' . ($secure ? 'secure' : 'unsecure') . '/base_link_url');
          $url = $this->_updatePathUseRewrites($url);
          break;

        case self::URL_TYPE_SKIN:
        case self::URL_TYPE_SKIN_BASE:
        case self::URL_TYPE_SKIN_CLOUD:
        case self::URL_TYPE_SKIN_MERGE:
          $secure = is_null($secure) ? $this->isCurrentlySecure() : (bool)$secure;
          $url = $this->getConfig('web/' . ($secure ? 'secure' : 'unsecure') . '/base_' . self::URL_TYPE_SKIN . '_url');
          if ($type != self::URL_TYPE_SKIN_BASE) {
            $cloudType = (self::URL_TYPE_SKIN == $type) ? 'image' : 'raw';
            $isCloud = (self::URL_TYPE_SKIN_CLOUD == $type) ? true : false;
            $rawUrl = $this->getCloudinaryUrl($cloudType, $isCloud);
            if (!is_numeric($rawUrl)) {
              $rawUrl = rtrim($rawUrl, DS);
              $url = $rawUrl . DS . self::URL_TYPE_SKIN;
            }
          }
          break;

        case self::URL_TYPE_JS:
          $secure = is_null($secure) ? $this->isCurrentlySecure() : (bool)$secure;
          $url = $this->getConfig('web/' . ($secure ? 'secure' : 'unsecure') . '/base_' . $type . '_url');
          if (!is_numeric($this->getCloudinaryUrl('raw')))
            $url = $this->getCloudinaryUrl('raw') . self::URL_TYPE_JS;
          break;

        case self::URL_TYPE_MEDIA:
        case self::URL_TYPE_MEDIA_MERGE:
        case self::URL_TYPE_MEDIA_BASE:
          $url = $this->_updateMediaPathUseRewrites($secure);
          if ($type != self::URL_TYPE_MEDIA_BASE) {
            $cloudType = (self::URL_TYPE_MEDIA_MERGE == $type) ? 'raw' : 'image';
            $cloudUrl = $this->getCloudinaryUrl($cloudType);
            if (!is_numeric($cloudUrl)){
              $cloudUrl = rtrim($cloudUrl, DS);
              $url = $cloudUrl . DS . self::URL_TYPE_MEDIA;
            }
          }
          break;

        default:
          throw Mage::exception('Mage_Core', Mage::helper('core')->__('Invalid base url type'));
      }

      if (false !== strpos($url, '{{base_url}}')) {
        $baseUrl = Mage::getConfig()->substDistroServerVars('{{base_url}}');
        $url = str_replace('{{base_url}}', $baseUrl, $url);
      }

      $this->_baseUrlCache[$cacheKey] = rtrim($url, '/') . '/';
    }

    return $this->_baseUrlCache[$cacheKey];
  }

  public function getCloudinaryUrl($type = null, $cloud = false)
  {
    if (!(int)$this->isCloudinaryActive())
      return 0;

    $baseJSCSS = rtrim($this->getConfig(self::XML_PATH_CLOUDINARY . 'basejscss'), '/');

    if ($type == 'raw' && $baseJSCSS && !$cloud)
      return $baseJSCSS . '/';

    $value = str_replace('{{type}}', $type, $this->getConfig(self::XML_PATH_CLOUDINARY . 'url'));
    $value = rtrim($value, '/');
    if ($type == 'image')
      $value = $value . '/';
    return $value;
  }

  public function isCloudinaryActive()
  {
    if (Mage::app()->getStore()->isAdmin())
      return 0;
    else
      return $this->getConfig(self::XML_PATH_CLOUDINARY . 'enable');
  }
}
