<?php
/**
* Magento
*
* NOTICE OF LICENSE
*
* This source file is subject to the Open Software License (OSL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/osl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@magentocommerce.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade Magento to newer
* versions in the future. If you wish to customize Magento for your
* needs please refer to http://www.magentocommerce.com for more information.
*
* @category    Mage
* @package     Mage_Catalog
* @copyright   Copyright (c) 2012 Magento Inc. (http://www.magentocommerce.com)
* @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
*/

/**
* Catalog product link model
*
* @category   Mage
* @package    Mage_Catalog
* @author      Magento Core Team <core@magentocommerce.com>
*/
class Onlinebiz_Optimize_Model_Product_Image extends Mage_Catalog_Model_Product_Image
{
	private function _mediaHelper()
	{
		return Mage::helper('onlinebiz_optimize');
	}

	/**
	* Set filenames for base file and new file
	*
	* @param string $file
	* @return Mage_Catalog_Model_Product_Image
	*/
	public function setBaseFile($file, $isAdvanced = false)
	{
		if( Mage::app()->getStore()->isAdmin() ){
			return parent::setBaseFile($file);
		}
		$this->_isBaseFilePlaceholder = false;
		$suffixParams = !$isAdvanced ? $this->_mediaHelper()->getIMGixOptions() : $this->_mediaHelper()->getIMGixAdvancedOptions();
		$disableIMGCache = $this->_mediaHelper()->isDisabledIMGCache();
		if (($file) && (0 !== strpos($file, '/', 0))) {
			$file = '/' . $file;
		}
		$baseDir = Mage::getSingleton('catalog/product_media_config')->getBaseMediaPath();

		if ('/no_selection' == $file) {
			$file = null;
		}
		if ($file) {
			if ((!$this->_fileExists($baseDir . $file)) || !$this->_checkMemory($baseDir . $file)) {
				$file = null;
			}
		}
		if (!$file) {
			// check if placeholder defined in config
			$isConfigPlaceholder = Mage::getStoreConfig("catalog/placeholder/{$this->getDestinationSubdir()}_placeholder");
			$configPlaceholder   = '/placeholder/' . $isConfigPlaceholder;
			if ($isConfigPlaceholder && $this->_fileExists($baseDir . $configPlaceholder)) {
				$file = $configPlaceholder;
			} else {
				// replace file with skin or default skin placeholder
				$skinBaseDir     = Mage::getDesign()->getSkinBaseDir();
				$skinPlaceholder = "/images/catalog/product/placeholder/{$this->getDestinationSubdir()}.jpg";
				$file = $skinPlaceholder;
				if (file_exists($skinBaseDir . $file)) {
					$baseDir = $skinBaseDir;
				} else {
					$baseDir = Mage::getDesign()->getSkinBaseDir(array('_theme' => 'default'));
					if (!file_exists($baseDir . $file)) {
						$baseDir = Mage::getDesign()->getSkinBaseDir(array('_theme' => 'default', '_package' => 'base'));
					}
				}
			}
			$this->_isBaseFilePlaceholder = true;
		}

		$baseFile = $baseDir . $file;

		if (((!$file) || (!file_exists($baseFile))) && !$this->_mediaHelper()->isDevMode()) {
			throw new Exception(Mage::helper('catalog')->__('Image file was not found.'));
		}

		$this->_baseFile = $baseFile;

		// build new filename (most important params)
		$path = array(
			Mage::getSingleton('catalog/product_media_config')->getBaseMediaPath(),
			'cache',
			Mage::app()->getStore()->getId(),
			$path[] = $this->getDestinationSubdir()
		);
		if($disableIMGCache){
			$path = array(Mage::getSingleton('catalog/product_media_config')->getBaseMediaPath());
		}
		if((!empty($this->_width)) || (!empty($this->_height))){
			if(!$this->_height)
				$this->_height = $this->_width;
			if($disableIMGCache){
				$suffixParams .= ($suffixParams != '' ) ? "&w={$this->_width}&h={$this->_height}" : "?w={$this->_width}&h={$this->_height}";
			} else {
				$path[] = "{$this->_width}x{$this->_height}";
			}
		}

		// add misk params as a hash
		$miscParams = array(
			($this->_keepAspectRatio  ? '' : 'non') . 'proportional',
			($this->_keepFrame        ? '' : 'no')  . 'frame',
			($this->_keepTransparency ? '' : 'no')  . 'transparency',
			($this->_constrainOnly ? 'do' : 'not')  . 'constrainonly',
			$this->_rgbToString($this->_backgroundColor),
			'angle' . $this->_angle,
			'quality' . $this->_quality
		);

		// if has watermark add watermark params to hash
		if ($this->getWatermarkFile()) {
			$miscParams[] = $this->getWatermarkFile();
			$miscParams[] = $this->getWatermarkImageOpacity();
			$miscParams[] = $this->getWatermarkPosition();
			$miscParams[] = $this->getWatermarkWidth();
			$miscParams[] = $this->getWatermarkHeigth();
		}
		if(!$disableIMGCache)
		$path[] = md5(implode('_', $miscParams));
		// append prepared filename
		$this->_newFile = implode('/', $path) . $file . $suffixParams; // the $file contains heading slash
		return $this;
	}

	/**
	* First check this file on FS
	* If it doesn't exist - try to download it from DB
	*
	* @param string $filename
	* @return bool
	*/
	protected function _fileExists($filename)
	{
		if($this->_mediaHelper()->isDevMode())
			return true;
		$filename = $this->_removeQueryFile($filename);
		if (file_exists($filename)) {
			return true;
		} else {
			return Mage::helper('core/file_storage_database')->saveFileToFilesystem($filename);
		}
	}

	/**
	* @return Mage_Catalog_Model_Product_Image
	*/
	public function saveFile()
	{
		$filename = $this->getNewFile();
		$filename = $this->_removeQueryFile($filename);
		$this->getImageProcessor()->save($filename);
		Mage::helper('core/file_storage_database')->saveFile($filename);
		return $this;
	}

	private function _removeQueryFile($filename)
	{
		$fileInfo = parse_url($filename);
		if(isset($fileInfo['query']))
		$filename = str_replace('?' .$fileInfo['query'], '', $filename);
		return $filename;
	}
}
